from .connection import Connection
from .activity import Activity


class Lama:
    def __init__(self, base_url: str, api_user: str, passwd: str, proxy_server=None, proxy_port=None):
        self._base_url = base_url
        self._connection = Connection(
            api_user, passwd, proxy_server=proxy_server, proxy_port=proxy_port)

    def _build_url(self, endpoint_url, params={}):
        return f'{self._base_url}{endpoint_url.format(**params)}'

    def activity(self):
        return Activity(self)



from re import findall


class Activity:
    __endpoints = {
        'activities': ('get', '/activities'),
        'activity': ('get', '/activities/{activityId}'),
        'cancel': ('post', '/activities/{activityId}/cancel'),
        'delete': ('delete', '/activities/{activityId}'),
        'hold': ('post', '/activities/{activityId}/hold'),
        'release': ('post', '/activities/{activityId}/release'),
        'retry': ('post', '/activities/{activityId}/retry'),
        'steps': ('get', '/activities/{activityId}/steps'),
        'step': ('get', '/activities/{activityId}/steps/{stepId}'),
        'step_logs': ('get', '/activities/{activityId}/steps/{stepId}/logs'),
        'continue': ('post', '/activities/{activityId}/steps/{stepId}/continue'),
        'create_activity': ('post', '/activities'),
    }

    def __init__(self, parent):
        self.__parent = parent
        self._build_url = parent._build_url
        self.connection = parent._connection

    def get_all(self):
        endpoint = self.__endpoints.get('activities')

        return self.connection.request(
            self._build_url(endpoint[1]),
            endpoint[0],
        )

    def get(self, activity_id):
        endpoint = self.__endpoints.get('activity')
        params = {'activityId': activity_id}

        return self.connection.request(
            self._build_url(endpoint[1], params),
            endpoint[0],
        )

    def execute_job_template(self, template_name, *, dry_run=False):
        endpoint = self.__endpoints.get('create_activity')
        data = {'type': 'operationTemplate',
                'operation': template_name,
                'validateOnly': dry_run}

        return self.connection.request(
            self._build_url(endpoint[1]),
            endpoint[0],
            data=data
        )

    def get_status(self, *, days=1, fields='items(description,status,beginTime)', user=None, dry_run=False):
        endpoint = self.__endpoints.get('activities')
        params = {
            'user': user,
            'days': days,
            'fields': fields
        }

        return self.connection.request(
            self._build_url(endpoint[1]),
            endpoint[0],
            params=params
        )

    def naive_request(self, endpoint, *, params={}, data={}, headers={}):
        if endpoint not in self.__endpoints.keys():
            raise ValueError(
                f'endpoint must be one of {self.__endpoints.keys()}')

        endpoint = self.__endpoints.get(endpoint)

        required_params = findall(r'\{(.*?)\}', endpoint[1])
        for param in required_params:
            if param not in params.keys():
                raise ValueError(
                    f'missing parameter "{param}"')

        return self.connection.request(
            self._build_url(endpoint[1], params),
            endpoint[0],
            data=data,
            headers=headers
        )



import json
import urllib.parse
from requests import Session


class Connection:
    _allowed_methods = ['get', 'post', 'delete']

    def __init__(self, api_user, passwd,  *, proxy_server=None, proxy_port=8080):
        self.__user = api_user
        self.__pass = passwd
        self.session = None
        self.proxy = {}
        self.set_proxy(proxy_server, proxy_port)

    def set_proxy(self, proxy_server, proxy_port):
        if proxy_server and proxy_port:
            self.proxy = {
                'http': 'http://{proxy_server}:{proxy_port}',
                'https': 'https://{proxy_server}:{proxy_port}'
            }

    def get_session(self):
        session = Session()
        session.auth = (self.__user, self.__pass)
        session.proxy = self.proxy

        return session

    def request(self, url, method, *, data={}, headers={}, params={}):
        if method not in self._allowed_methods:
            raise ValueError(f'method must be one of {self._allowed_methods}')

        if method == 'post':
            headers['x-csrf-token'] = ''
            headers['Content-Type'] = 'application/json'

        if self.session is None:
            self.session = self.get_session()

        response = self.session.request(
            method,
            f'{url}?{urllib.parse.urlencode(params)}',
            data=json.dumps(data),
            headers=headers)
        response.raise_for_status()

        try:
            response = response.json()
        except json.decoder.JSONDecodeError:
            response = response.text

        return response



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

CLASSIFIERS = [
    'Development Status :: 2 - Pre-Alpha',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: Apache Software License',
    'Topic :: Software Development :: Libraries',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3 :: Only',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.7',
    'Operating System :: OS Independent',
]

requires = [
    'requests',
]

setup(
    name='SAPLaMa',
    version='0.0.5',
    packages=find_packages(),
    url='https://git.tech.rz.db.de/sapzs/python_sap_lama_api',
    license='Apache License 2.0',
    author='Maximilian Wiederer',
    author_email='maximilian.wiederer@deutschebahn.com',
    maintainer='Maximilian Wiederer',
    maintainer_email='maximilian.wiederer@deutschebahn.com',
    description='SAP LAMA API made easy',
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=CLASSIFIERS,
    python_requires=">=3.7",
    install_requires=requires,
)
